/**********************************************************
* Kevin Yang
***********************************************************
* TEJ 4M0
***********************************************************
* Date: January 21, 2022
***********************************************************
* Description: Sad Cat Fixer with Servos and laser(LED)
* controlled with IR remote. Also an ultrasonic distance sensor that
* controls a buzzer when an object gets close enough to the circuit.
***********************************************************
* Digital Pins Used: 2, 5, 6, 8, 9, 10, 13
**********************************************************/

#include <IRremote.h> // include the IR Remote library
#include <Servo.h> // include the Servo library

#define left 0xFF02FD // hexcode for 'left button'
#define right 0xFFC23D // hexcode for 'right button'
#define elevation 0xFF906F // hexcode for 'up button'
#define depression 0xFFA857 // hexcode for 'down button'
#define holdDown 0xFFFFFFFF // hexcode for when a button is held down
#define power 0xFFA25D // hexcode for 'power button'

Servo xServo; // create servo object for horizontal servo
Servo yServo; // create servo object for vertical servo

const int RECV_PIN = 2; // IR receiver pin

// IR Receiver Setup
IRrecv irrecv(RECV_PIN);
decode_results results;

int xServoAngle = 90;
int xAngleDelta = 0;
int yServoAngle = 90;
int yAngleDelta = 0;

// Ultrasonic Sensor Pins
const int echoPin = 6;
const int trigPin = 5;

const int buzzer = 13; // buzzer pin
const int laser = 8; // LED/Laser pin

long duration; // duration for ultrasnoic sensor
int distance; // distance found by ultrasonic sensor

bool horizontal, vertical; // logic to help IR remote control Servos
bool lightON = false; // logic to help IR remote control LED/Laser

/**********************************************************
* void setup
**********************************************************/
void setup()
{
  pinMode(2, INPUT); // IR receiver pin
  pinMode(3, OUTPUT); // horizontal Servo pin
  pinMode(4, OUTPUT); // vertical Servo pin

  pinMode(echoPin, INPUT); // Sets the echoPin as an Input
  pinMode(trigPin, OUTPUT); // Sets the trigPin as an Output
  
  Serial.begin(9600); // initialize serial monitor

  // initialize IR receiver
  irrecv.enableIRIn();
  irrecv.blink13(true);
  
  xServo.attach(9); // attach horizontal servo to pin 9
  xServo.write(xServoAngle); // set servo to neutral angle (90 degrees)
  
  yServo.attach(10); // attach horizontal servo to pin 10
  yServo.write(yServoAngle); // set servo to neutral angle (90 degrees)
  
  delay(1000);
}
/**********************************************************/

/**********************************************************
* void loop
**********************************************************/
void loop()
{
  if (irrecv.decode(&results))
  {
    Serial.println(results.value, HEX);
    irrecv.resume();
    
    delay(100);
  }
  
  // horizontal Servo
  if(results.value == left) // left button
  {
    xAngleDelta = 10; // change in angle for each button click
    xServoAngle += xAngleDelta; // incrementing the angle of the servo
    xServo.write(xServoAngle); // write servo to new angle
    
    #define lastButton 0xFF02FD // record the left button as last button pressed

    // print angle for debugging
    Serial.print("xAngleDelta = ");
    Serial.println(xAngleDelta);
    
    horizontal = true; // last servo used was the horizontal servo
    vertical = false;
  }
  
  else if(results.value == right) // right button
  {
    xAngleDelta = -10; // change in angle for each button click
    xServoAngle += xAngleDelta;  // incrementing the angle of the servo
    xServo.write(xServoAngle); // write servo to new angle
    
    #define lastButton 0xFFC23D // record the right button as last button pressed

    // print angle for debugging
    Serial.print("xAngleDelta = ");
    Serial.println(xAngleDelta);
    
    horizontal = true; // last servo used was the horizontal servo
    vertical = false;
  }
  
  else if(results.value == holdDown && horizontal &&((lastButton == left) || (lastButton == right))) // when a button for horizontal servo is held down
  {
    xServoAngle += xAngleDelta; // incrementing the angle of the servo
    xServo.write(xServoAngle); // write servo to new angle

    // print angle for debugging
    Serial.print("xAngleDelta = ");
    Serial.println(xAngleDelta);
  }
  
  // vertical Servo
  else if(results.value == elevation) // plus button
  {
    yAngleDelta = 10; // change in angle for each button click
    yServoAngle += yAngleDelta; // incrementing the angle of the servo
    
    yServo.write(yServoAngle); // write servo to new angle
    
    #define lastButton 0xFF906F // record the up button as last button pressed

    // print angle for debugging
    Serial.print("yAngleDelta = ");
    Serial.println(yAngleDelta);
    
    horizontal = false;
    vertical = true; // last servo used was the vertical servo
  }
  
  else if(results.value == depression) // minus button
  {
    yAngleDelta = -10; // change in angle for each button click
    yServoAngle += yAngleDelta; // incrementing the angle of the servo
    
    yServo.write(yServoAngle); // write servo to new angle
    
    #define lastButton 0xFFA857 // record the up button as last button pressed

    // print angle for debugging
    Serial.print("yAngleDelta = ");
    Serial.println(yAngleDelta);
    
    horizontal = false;
    vertical = true; // last servo used was the vertical servo
  }
  
  else if (results.value == holdDown && vertical && ((lastButton == elevation) || (lastButton == depression))) // when a button is held down for the vertical servo
  {
    yServoAngle += yAngleDelta; // incrementing the angle of the servo
    yServo.write(yServoAngle); // write servo to new angle

    // print angle for debugging
    Serial.print("yAngleDelta = ");
    Serial.println(yAngleDelta);
  }
  
  // when servo angles are outside of range of motion
  if(xServoAngle > 180) // when horizontal angle goes over 180 degrees, keep it at 180 degrees
  {
    xServoAngle = 180;
  }
  else if(xServoAngle < 0) // when horizontal angle goes below 0 degrees, keep it at 0 degrees
  {
    xServoAngle = 0;
  }
  else if(yServoAngle > 180) // when vertical angle goes over 180 degrees, keep it at 180 degrees
  {
    yServoAngle = 180;
  }
  else if(yServoAngle < 0) // when vertical angle goes below 0 degrees, keep it at 0 degrees
  {
    yServoAngle = 0;
  }

  // print data for debugging
  Serial.print("xServoAngle = "); // horizontal servo
  Serial.println(xServoAngle);
  Serial.print("yServoAngle = "); // vertical servo
  Serial.println(yServoAngle);
  Serial.print("lastButton = "); // last button pressed
  Serial.println(lastButton);
  Serial.println(""); // line for spacing

  if((results.value == power) && (lightON == false)) // when the power button is pressed and the LED/Laser is off
  {
    digitalWrite(laser, HIGH); // turn LED/Laser on
    lightON = true; // record the light was turned on
  }
  else if((results.value == power) && (lightON == true)) // when the power button is pressed and the LED/Laser is on
  {
    digitalWrite(laser, LOW); // turn LED/Laser off
    lightON = false; // record the light was turned off
  }

  // clear IR receiver
  results.value = 0;
  #define lastButton 0

  ultrasonicCycle(); // use ultrasonic sensor to check distance

  if(distance <= 30) // when cat gets closer than 30 cm, sound buzzer
  {
    digitalWrite(buzzer, HIGH); // should use 'tone' at 500-1000 Hz, does not compile when using 'tone' command [tone(buzzer, 500)]
  }
  else // when cat gets further than 30 cm, turn off buzzer
  {
    digitalWrite(buzzer, LOW);  // should use noTone
  }
}
/**********************************************************/

/**********************************************************
* void ultrasonicCycle
**********************************************************/
void ultrasonicCycle() // the process the ultrasonic sensor uses to detect distance
{
    // Clears the trigPin
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  // Sets the trigPin on HIGH state for 10 micro seconds
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  // Reads the echoPin, returns the sound wave travel time in microseconds
  duration = pulseIn(echoPin, HIGH);
  // Calculating the distance
  distance = duration*0.034/2;
  // Prints the distance on the Serial Monitor
  Serial.print("Distance: ");
  Serial.println(distance);
}
/**********************************************************/
